@@ -62,7 +62,16 @@ const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
 
 
+// maximum helper
+
+inline float samplv1_max ( float a, float b )
+{
+	return (a > b ? a : b);
+}
+
+
 // hyperbolic-tangent fast approximation
+
 inline float samplv1_tanhf ( const float x )
 {
 	const float x2 = x * x;
@@ -119,6 +128,14 @@ inline float samplv1_pow2f ( const float x )
 }
 
 
+// convert note to frequency (hertz)
+
+inline float samplv1_freq ( float note )
+{
+	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
+}
+
+
 // envelope
 
 struct samplv1_env
@@ -531,7 +548,7 @@ public:
 		m_t1 = m_b3; m_b3 = (m_b2 + m_t2) * m_p - m_b3 * m_f;
 
 		m_b4 = (m_b3 + m_t1) * m_p - m_b4 * m_f;
-	//	m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	-- clipping
+		m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	// clipping
 
 		m_b0 = input;
 
@@ -609,9 +626,9 @@ public:
 
 protected:
 
-	virtual float evaluate(uint16_t i)
+	float evaluate(uint16_t i)
 	{
-		samplv1_ramp3::evaluate(i);
+		samplv1_ramp3::update();
 
 		const float wpan = 0.25f * M_PI
 			* (1.0f + m_param1_v)
@@ -623,18 +640,30 @@ protected:
 };
 
 
+// pressure smoother (3 parameters)
 
-// convert note to frequency (hertz)
-
-inline float note_freq ( float note )
+class samplv1_pre : public samplv1_ramp3
 {
-	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
-}
+public:
+
+	samplv1_pre() : samplv1_ramp3() {}
+
+protected:
+
+	float evaluate(uint16_t)
+	{
+		samplv1_ramp3::update();
+
+		return m_param1_v * samplv1_max(m_param2_v, m_param3_v);
+	}
+};
 
 
 // forward decl.
+
 class samplv1_impl;
 
+
 // voice
 
 struct samplv1_voice : public samplv1_list<samplv1_voice>
@@ -646,13 +675,14 @@ struct samplv1_voice : public samplv1_list<samplv1_voice>
 
 	int note;									// voice note
 	float vel;									// velocity to vol
+	float pre;									// key pressure/aftertouch
 
 	float gen1_freq;							// frequency and phase
 
 	float lfo1_sample;
 
 	samplv1_filter1 dcf11, dcf12;				// filters
-	samplv1_filter1 dcf13, dcf14;
+	samplv1_filter2 dcf13, dcf14;
 
 	samplv1_env::State dca1_env;				// envelope states
 	samplv1_env::State dcf1_env;
@@ -660,6 +690,8 @@ struct samplv1_voice : public samplv1_list<samplv1_voice>
 
 	samplv1_glide gen1_glide;					// glides (portamento)
 
+	samplv1_pre dca1_pre;
+
 	bool sustain;
 };
 
@@ -752,8 +784,6 @@ private:
 	samplv1_pan   m_pan1;
 	samplv1_ramp4 m_vol1;
 
-	samplv1_ramp2 m_pre1;
-
 	samplv1_fx_chorus   m_chorus;
 	samplv1_fx_flanger *m_flanger;
 	samplv1_fx_phaser  *m_phaser;
@@ -915,7 +945,7 @@ void samplv1_impl::setSampleFile ( const char *pszSampleFile )
 
 	if (pszSampleFile) {
 		m_gen1.sample0 = *m_gen1.sample;
-		gen1_sample.open(pszSampleFile, note_freq(m_gen1.sample0));
+		gen1_sample.open(pszSampleFile, samplv1_freq(m_gen1.sample0));
 	}
 }
 
@@ -1112,20 +1142,22 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			const float vel = float(value) / 127.0f;
 			// quadratic velocity law
 			pv->vel = samplv1_velocity(vel * vel, *m_def1.velocity);
+			// pressure/aftertouch
+			pv->pre = 0.0f;
+			pv->dca1_pre.reset(m_def1.pressure, &m_ctl.pressure, &pv->pre);
 			// generate
 			pv->gen1.start();
 			// frequencies
 			const float freq1 = float(key)
 				+ *m_gen1.octave * OCTAVE_SCALE
 				+ *m_gen1.tuning * TUNING_SCALE;
-			pv->gen1_freq = note_freq(freq1);
+			pv->gen1_freq = samplv1_freq(freq1);
 			// filters
-			const samplv1_filter1::Type type1
-				= samplv1_filter1::Type(int(*m_dcf1.type));
-			pv->dcf11.reset(type1);
-			pv->dcf12.reset(type1);
-			pv->dcf13.reset(type1);
-			pv->dcf14.reset(type1);
+			const int type1 = int(*m_dcf1.type);
+			pv->dcf11.reset(samplv1_filter1::Type(type1));
+			pv->dcf12.reset(samplv1_filter1::Type(type1));
+			pv->dcf13.reset(samplv1_filter2::Type(type1));
+			pv->dcf14.reset(samplv1_filter2::Type(type1));
 			// envelopes
 			m_dcf1.env.start(&pv->dcf1_env);
 			m_lfo1.env.start(&pv->lfo1_env);
@@ -1157,6 +1189,12 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			}
 		}
 	}
+	// key pressure/poly.aftertouch
+	else if (status == 0xa0) {
+		samplv1_voice *pv = m_notes[key];
+		if (pv && pv->note >= 0)
+			pv->pre = *m_def1.pressure * float(value) / 127.0f;
+	}
 	// control change
 	else if (status == 0xb0) {
 		switch (key) {
@@ -1278,8 +1316,6 @@ void samplv1_impl::reset (void)
 	m_pan1.reset(m_out1.panning, &m_ctl.panning, &m_aux1.panning);
 	m_wid1.reset(m_out1.width);
 
-	m_pre1.reset(m_def1.pressure, &m_ctl.pressure);
-
 	// flangers
 	if (m_flanger == 0)
 		m_flanger = new samplv1_fx_flanger [m_iChannels];
@@ -1328,7 +1364,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	if (m_gen1.sample0 != *m_gen1.sample) {
 		m_gen1.sample0  = *m_gen1.sample;
-		gen1_sample.reset(note_freq(m_gen1.sample0));
+		gen1_sample.reset(samplv1_freq(m_gen1.sample0));
 	}
 
 	if (bool(int(*m_gen1.loop)) != gen1_sample.isLoop())
@@ -1337,12 +1373,6 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	if (int(*m_lfo1.shape) != int(lfo1_wave.shape()) || *m_lfo1.width != lfo1_wave.width())
 		lfo1_wave.reset(samplv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
 
-	m_wid1.process(nframes);
-	m_pan1.process(nframes);
-	m_vol1.process(nframes);
-
-	m_pre1.process(nframes);
-
 	// per voice
 
 	samplv1_voice *pv = m_play_list.next();
@@ -1376,7 +1406,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				// velocities
 
 				const float vel1
-					= (pv->vel + (1.0f - pv->vel) * m_pre1.value(j));
+					= (pv->vel + (1.0f - pv->vel) * pv->dca1_pre.value(j));
 
 				// generators
 
@@ -1402,11 +1432,12 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float reso1 = samplv1_sigmoid_1(*m_dcf1.reso
 					* env1 * (1.0f + *m_lfo1.reso * lfo1));
 
-				gen1 = pv->dcf11.output(gen1, cutoff1, reso1);
-				gen2 = pv->dcf12.output(gen2, cutoff1, reso1);
 				if (int(*m_dcf1.slope) > 0) { // 24db/octave
 					gen1 = pv->dcf13.output(gen1, cutoff1, reso1);
 					gen2 = pv->dcf14.output(gen2, cutoff1, reso1);
+				} else { // 12db/octave
+					gen1 = pv->dcf11.output(gen1, cutoff1, reso1);
+					gen2 = pv->dcf12.output(gen2, cutoff1, reso1);
 				}
 
 				// volumes
@@ -1435,6 +1466,10 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			nblock -= ngen;
 
+			// voice ramps countdown
+
+			pv->dca1_pre.process(ngen);
+
 			// envelope countdowns
 
 			if (pv->dca1_env.running && pv->dca1_env.frames == 0)
@@ -1486,6 +1521,12 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				*out++ = samplv1_sigmoid(*in++);
 		}
 	}
+
+	// post-processing
+
+	m_wid1.process(nframes);
+	m_pan1.process(nframes);
+	m_vol1.process(nframes);
 }
 
 
