@@ -522,6 +522,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
 	m_ui.StatusBar->setModified(false);
+	m_ui.Preset->setDirtyPreset(false);
 }
 
 
@@ -578,10 +579,8 @@ void samplv1widget::paramChanged ( float fValue )
 			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
-		m_ui.StatusBar->setModified(true);
+		updateDirtyPreset(true);
 	}
-
-	m_ui.Preset->dirtyPreset();
 }
 
 
@@ -630,7 +629,7 @@ void samplv1widget::resetParams (void)
 	}
 
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -657,11 +656,9 @@ void samplv1widget::swapParams ( bool bOn )
 		}
 	}
 
-	m_ui.Preset->dirtyPreset();
-
 	const bool bSwapA = m_ui.SwapParamsAButton->isChecked();
 	m_ui.StatusBar->showMessage(tr("Swap %1").arg(bSwapA ? 'A' : 'B'), 5000);
-	m_ui.StatusBar->setModified(true);
+	updateDirtyPreset(true);
 }
 
 
@@ -727,7 +724,7 @@ void samplv1widget::newPreset (void)
 	resetParamValues();
 
 	m_ui.StatusBar->showMessage(tr("New preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 
 	m_ui.Gen1Sample->openSample();
 }
@@ -808,7 +805,7 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 	m_ui.Preset->setPreset(sPreset);
 
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 
 	QDir::setCurrent(currentDir.absolutePath());
 }
@@ -849,7 +846,7 @@ void samplv1widget::savePreset ( const QString& sFilename )
 	}
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -952,9 +949,7 @@ void samplv1widget::clearSample (void)
 	clearSampleFile();
 
 	m_ui.StatusBar->showMessage(tr("Clear sample"), 5000);
-	m_ui.StatusBar->setModified(true);
-
-	m_ui.Preset->dirtyPreset();
+	updateDirtyPreset(true);
 }
 
 
@@ -964,9 +959,7 @@ void samplv1widget::loadSample ( const QString& sFilename )
 	loadSampleFile(sFilename);
 
 	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
-	m_ui.StatusBar->setModified(true);
-
-	m_ui.Preset->dirtyPreset();
+	updateDirtyPreset(true);
 }
 
 
@@ -1030,7 +1023,7 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 	--m_iUpdate;
 
 	if (pSample && bDirty)
-		m_ui.Preset->dirtyPreset();
+		updateDirtyPreset(true);
 }
 
 
@@ -1121,10 +1114,9 @@ void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
 		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
 		if (bDirty) {
-			m_ui.Preset->dirtyPreset();
 			m_ui.StatusBar->showMessage(tr("Loop start: %1, end: %2")
 				.arg(iLoopStart).arg(iLoopEnd), 5000);
-			m_ui.StatusBar->setModified(true);
+			updateDirtyPreset(true);
 		}
 	} else {
 		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
@@ -1217,4 +1209,12 @@ void samplv1widget::helpAboutQt (void)
 }
 
 
+// Dirty flag (overridable virtual) methods.
+void samplv1widget::updateDirtyPreset ( bool bDirtyPreset )
+{
+	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.Preset->setDirtyPreset(bDirtyPreset);
+}
+
+
 // end of samplv1widget.cpp
